package ecfdemo.sharedObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.TimeoutException;

import org.eclipse.ecf.core.identity.ID;
import org.eclipse.ecf.core.jobs.JobsExecutor;
import org.eclipse.ecf.core.sharedobject.BaseSharedObject;
import org.eclipse.ecf.core.sharedobject.ReplicaSharedObjectDescription;
import org.eclipse.ecf.core.sharedobject.SharedObjectInitException;
import org.eclipse.ecf.core.sharedobject.SharedObjectMsg;

/**
 * 
 * @author jungeun07.park
 *
 */
public abstract class AbstractSharedZObject extends BaseSharedObject {
	
	private static final String REPLICATE_OBJECT_TO_REMOTE = "replicateObjectToRemote";
	
	private static final String CALL_REQUEST = "handleCallRequest";
	private static final String CALL_RESPONSE = "handleCallResponse";
	
	private static final String DEFAULT_EXECUTOR_TYPE = "threads";
	
	private static final long RESPONSE_WAIT_INTERVAL = 5000;
	
	private ID remoteClient;
	
	protected AbstractSharedZObject() {
		super();
	}
	
	
	protected AbstractSharedZObject(ID remoteClient) {
		super();
		this.remoteClient = remoteClient;
	}
	
	
	protected ID getRemoteClient() {
		return remoteClient;
	}
	
	
	protected void initialize() throws SharedObjectInitException {
		super.initialize();
	}
	
	
	protected abstract ReplicaSharedObjectDescription getReplicaDescription(ID receiver);
	
	
	protected void replicateObjectToRemote(ID remoteContainers) {
		if (isPrimary() && isConnected()) {
			AbstractSharedZObject.this.replicateToRemoteContainers(new ID[] {remoteContainers});
		}
	}
	
	
	public void sendReplicateMsgToSelf(ID targetId) {
		SharedObjectMsg msg 
			= SharedObjectMsg.createMsg(REPLICATE_OBJECT_TO_REMOTE, targetId);
		sendSharedObjectMsgToSelf(msg);
	}
	
	
	protected boolean handleSharedObjectMsg(SharedObjectMsg msg) {
		try {
			msg.invoke(this);
			return true;
			
		} catch (Exception e) {
			System.out.println("Exception Occured While invoking msg");
		}
		
		return false;
	}
	
	
	private void sendCallRequest(ID targetID,String messgae) throws IOException {
		
		try {
			sendSharedObjectMsgTo(targetID, SharedObjectMsg.createMsg(CALL_REQUEST, messgae));
			
		} catch (final IOException e) {
			throw e;
		}
		
	}
	
	
}

