package ecfgenericclient.localclient;


import org.eclipse.ecf.core.ContainerCreateException;
import org.eclipse.ecf.core.ContainerFactory;
import org.eclipse.ecf.core.IContainer;
import org.eclipse.ecf.core.IContainerListener;
import org.eclipse.ecf.core.events.IContainerConnectedEvent;
import org.eclipse.ecf.core.events.IContainerEjectedEvent;
import org.eclipse.ecf.core.events.IContainerEvent;
import org.eclipse.ecf.core.identity.ID;
import org.eclipse.ecf.core.identity.IDFactory;
import org.eclipse.ecf.core.sharedobject.ISharedObject;
import org.eclipse.ecf.core.sharedobject.ISharedObjectContainer;
import org.eclipse.ecf.core.sharedobject.ISharedObjectManager;
import org.eclipse.ecf.core.sharedobject.events.ContainerSharedObjectMessageReceivingEvent;
import org.eclipse.ecf.core.sharedobject.events.ContainerSharedObjectMessageSendingEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectActivatedEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectDeactivatedEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectMessageEvent;
import org.eclipse.ecf.core.sharedobject.events.SharedObjectManagerAddEvent;

import ecfdemo.sharedObject.MySharedObject;

public class ECFClient {

	public static final String GENERIC_CLIENT_CONTAINER = "ecf.generic.client";
	public static String containerId = "client";
	private static final int CLIENT_INTERVAL = 30000;

	public static final String SHARED_HOST_ID = "ZSHARED_HOST:";
	private IContainer soContainer;
	private ID targetID;
	private String remoteHostId;
	private Thread connectThread;
	boolean isConnected = true;
	private MySharedObject sharedObject;
	

	public ECFClient(String remoteHostId) {
		this.remoteHostId = remoteHostId;
		createClient();	
		this.connectThread = new Thread() {
			public void run() {
				while (isConnected) {
					try {
						soContainer.connect(targetID, null);				
						Thread.sleep(CLIENT_INTERVAL);
						isConnected = false;
					} catch (Exception e) {
						if (soContainer.getConnectedID() != null) {
							soContainer.disconnect();
						} else {
						}

						try { 
							Thread.sleep(CLIENT_INTERVAL);
						} catch (InterruptedException ie) {
						}
					} 
				}

				try {
					Thread.sleep(2000);
				} catch (InterruptedException e) {
				}

			}
		};
	}
	
	public void connectClient(){
		this.connectThread.start();
	}

	private void createClient() {
		try {
			soContainer = ContainerFactory.getDefault()
					.createContainer(GENERIC_CLIENT_CONTAINER, containerId);

			// ADD Container Listener
			soContainer.addListener(createContainerListener());

			targetID = IDFactory.getDefault().createStringID(remoteHostId);

		} catch (ContainerCreateException e) {

		}
	}

	private IContainerListener createContainerListener() {
		return new IContainerListener() {
			public void handleEvent(IContainerEvent evt) {
				if (evt instanceof IContainerEjectedEvent) {
					System.out.println(" client IContainerEjectedEvent occured, " + evt);

				} else if (evt instanceof IContainerConnectedEvent) {
					System.out.println("[zsharedobject] client "
							+ "IContainerConnectedEvent occured, " + evt);
				} else if (evt instanceof ISharedObjectActivatedEvent) {
					System.out.println(" client ISharedObjectActivatedEvent occured, " + evt);

					ID activatedId 
					= ((ISharedObjectActivatedEvent) evt).getActivatedID();
					registerRemoteClient(activatedId);

				} else if (evt instanceof ISharedObjectDeactivatedEvent) {
					System.out.println("[zsharedobject] client "
							+ "ISharedObjectDeactivatedEvent occured, " + evt);
				} else if (evt instanceof ISharedObjectMessageEvent) {
					System.out.println("[zsharedobject] client "
							+ "ISharedObjectMessageEvent occured, " + evt);
				} else if (evt instanceof ContainerSharedObjectMessageSendingEvent) {
					System.out.println("[zsharedobject] client "
							+ "ContainerSharedObjectMessageSendingEvent occured, " + evt);
				} else if (evt instanceof ContainerSharedObjectMessageReceivingEvent) {
					System.out.println("[zsharedobject] client "
							+ "ContainerSharedObjectMessageReceivingEvent occured, " + evt);
				} else if (evt instanceof SharedObjectManagerAddEvent) {
					System.out.println("[zsharedobject] client "
							+ "SharedObjectManagerAddEvent occured, " + evt);
				} else {
					System.out.println("[zsharedobject] client other event occured (" + evt + ")");
				}
			}
		};
	}	

	private void registerRemoteClient(ID sharedObjectId) {		
		if (sharedObjectId.getName().indexOf(SHARED_HOST_ID) == -1) {	
			return;
		}

		sharedObject = (MySharedObject) getSharedObject(sharedObjectId);
		isConnected = true;

		System.out.println( " client ID(" + soContainer.getID().getName() 
				+ ") connect OK !!!");	
	}

	private ISharedObject getSharedObject(ID sharedObjectId) {
		ISharedObjectManager soManager 
		= ((ISharedObjectContainer) soContainer).getSharedObjectManager();
		ISharedObject sharedObject = soManager.getSharedObject(sharedObjectId);

		return sharedObject;
	}


}

