package ecfdemo.sharedcontainer;

import org.eclipse.ecf.core.ContainerCreateException;
import org.eclipse.ecf.core.ContainerFactory;
import org.eclipse.ecf.core.IContainer;
import org.eclipse.ecf.core.IContainerListener;
import org.eclipse.ecf.core.events.IContainerConnectedEvent;
import org.eclipse.ecf.core.events.IContainerDisconnectedEvent;
import org.eclipse.ecf.core.events.IContainerEjectedEvent;
import org.eclipse.ecf.core.events.IContainerEvent;
import org.eclipse.ecf.core.identity.ID;
import org.eclipse.ecf.core.identity.IDCreateException;
import org.eclipse.ecf.core.identity.IDFactory;
import org.eclipse.ecf.core.sharedobject.ISharedObjectContainer;
import org.eclipse.ecf.core.sharedobject.ISharedObjectManager;
import org.eclipse.ecf.core.sharedobject.SharedObjectAddException;
import org.eclipse.ecf.core.sharedobject.events.ContainerSharedObjectMessageReceivingEvent;
import org.eclipse.ecf.core.sharedobject.events.ContainerSharedObjectMessageSendingEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectActivatedEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectDeactivatedEvent;
import org.eclipse.ecf.core.sharedobject.events.ISharedObjectMessageEvent;
import org.eclipse.ecf.core.sharedobject.events.SharedObjectManagerAddEvent;

import ecfdemo.sharedObject.MySharedObject;

public class SharedContainer extends AbstractSharedContainer {

	public static final String GENERIC_SERVER_CONTAINER = "ecf.generic.server";
	public static final String GENERIC_SERVER_ID = "ecftcp://localhost:3282/zserver";
	public static final String SHARED_HOST_ID = "ZSHARED_HOST:";

	private IContainer soContainer;
	private ID sharedHostID;

	public SharedContainer(){

	}

	@Override
	public void addSharedObject() throws Exception {
		sharedHostID = IDFactory.getDefault().createStringID(SHARED_HOST_ID);
			
		MySharedObject sharedZHostObject 
				= new MySharedObject("Please");
		
		addSharedObjectToContainer(sharedHostID, sharedZHostObject);
			
	}

	@Override
	public void startServer() throws Exception {
		try {
			soContainer = ContainerFactory.getDefault().createContainer(GENERIC_SERVER_CONTAINER, 
					IDFactory.getDefault().createStringID(GENERIC_SERVER_ID));
			soContainer.addListener(createConnectedContainerListener());

		} catch (ContainerCreateException e) {
			e.printStackTrace();
		} catch (IDCreateException e) {
			e.printStackTrace();
		} 
	}

	private IContainerListener createConnectedContainerListener() {
		return new IContainerListener() {
			public void handleEvent(IContainerEvent evt) {
				if (evt instanceof IContainerDisconnectedEvent 
						|| evt instanceof IContainerEjectedEvent) {
					System.out.println("[zsharedobject] server " 
							+ "IContainerDisconnectedEvent occured, " + evt);		
				} else if (evt instanceof IContainerConnectedEvent) {
					try {
						ID targetId = ((IContainerConnectedEvent) evt).getTargetID();
						System.out.println( " server IContainerConnectedEvent "
								+ "occured, conntected from " + targetId.getName());

						replicaHostToClient(targetId);	

					} catch (Exception e) {
						e.printStackTrace();
					}			
				} else if (evt instanceof ISharedObjectActivatedEvent) {
					System.out.println("[zsharedobject] server "
							+ "ISharedObjectActivatedEvent occured, " + evt);
				} else if (evt instanceof ISharedObjectDeactivatedEvent) {
					System.out.println("[zsharedobject] server "
							+ "ISharedObjectDeactivatedEvent occured, " + evt);
				} else if (evt instanceof ISharedObjectMessageEvent) {
					System.out.println("[zsharedobject] server "
							+ "ISharedObjectMessageEvent occured, " + evt);
				} else if (evt instanceof ContainerSharedObjectMessageSendingEvent) {
					System.out.println("[zsharedobject] server "
							+ "ContainerSharedObjectMessageSendingEvent occured, " + evt);
				} else if (evt instanceof ContainerSharedObjectMessageReceivingEvent) {
					System.out.println("[zsharedobject] server "
							+ "ContainerSharedObjectMessageReceivingEvent occured, " + evt);
				} else if (evt instanceof SharedObjectManagerAddEvent) {
					System.out.println("[zsharedobject] server "
							+ "SharedObjectManagerAddEvent occured, " + evt);
				} else {
					System.out.println("[zsharedobject] server other event occured (" + evt + ")");
				}
			}
		};
	}

	private void replicaHostToClient(final ID targetId) {

		new Thread("Thread replicaHostToClient") {
			public void run() {
				ISharedObjectManager soManager 
				= ((ISharedObjectContainer) soContainer).getSharedObjectManager();
				MySharedObject sharedZObject 
				= (MySharedObject) soManager.getSharedObject(sharedHostID);
				sharedZObject.sendReplicateMsgToSelf(targetId);
			}
		} .start();
	}
	
   private void addSharedObjectToContainer(ID sharedId, MySharedObject object) {
		
		ISharedObjectManager soManager 
			= ((ISharedObjectContainer) soContainer).getSharedObjectManager();
		
		if (checkSharedObjectNotExist(soManager, sharedId)) {
			try {
				soManager.addSharedObject(sharedId, object, null);
				
			} catch (SharedObjectAddException e) {
				e.printStackTrace();
			}
			
		} else {
			System.out.println("[zsharedobject] server : The shared Object is already exist : " 
				+ sharedId.getName());
		}	
	}
   
   public static boolean checkSharedObjectNotExist(ISharedObjectManager soManager, ID objectID) {
		boolean isNotExist = true;
		ID[] sharedObjectIdArr = soManager.getSharedObjectIDs();
		for (int i = 0; i < sharedObjectIdArr.length; i++) {
			ID sharedObjectId = sharedObjectIdArr[i];
			if ((sharedObjectId.getName()).equals(objectID.getName())) {
				isNotExist = false;
				break;
			}
		}			
		
		return isNotExist;
	}
	
}
